package MooseX::AttributeHelpers::Number;
use Moose;
use MooseX::AttributeHelpers::Sugar;

extends 'MooseX::AttributeHelpers::Base';

our $VERSION   = '0.02';
our $AUTHORITY = 'cpan:STEVAN';

define_attribute_helper (
    helper_type      => 'Num',
    method_provider  => 'MooseX::AttributeHelpers::MethodProvider::Number',
    shortcut         => 'Number',
);

no Moose;
no MooseX::AttributeHelpers::Sugar;

1;

__END__

=pod

=head1 NAME

MooseX::AttributeHelpers::Number

=head1 SYNOPSIS
  
  package Real;
  use Moose;
  use MooseX::AttributeHelpers;
  
  has 'integer' => (
      metaclass => 'Number',
      is        => 'ro',
      isa       => 'Int',
      default   => sub { 5 },
      provides  => {
          set => 'set',
          add => 'add',
          sub => 'sub',
          mul => 'mul',
          div => 'div',
          mod => 'mod',
          abs => 'abs',
      }
  );

  my $real = Real->new();
  $real->add(5); # same as $real->integer($real->integer + 5);
  $real->sub(2); # same as $real->integer($real->integer - 2);  
  
=head1 DESCRIPTION

This provides a simple numeric attribute, which supports most of the
basic math operations.  It is important to note that all operations modify the
value of the attribute in place.

=head1 METHOD PROVIDER

The methods for this metaclass are provided by
L<MooseX::AttributeHelpers::MethodProvider::Number>.

=head1 BUGS

All complex software has bugs lurking in it, and this module is no 
exception. If you find a bug please either email me, or add the bug
to cpan-RT.

=head1 AUTHOR

Robert Boone

=head1 COPYRIGHT AND LICENSE

Copyright 2007-2008 by Infinity Interactive, Inc.

L<http://www.iinteractive.com>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
