
package MooseX::AttributeHelpers::String;
use Moose;
use MooseX::AttributeHelpers::Sugar;

extends 'MooseX::AttributeHelpers::Base';

our $VERSION   = '0.01';
our $AUTHORITY = 'cpan:STEVAN';

define_attribute_helper (
    default_options  => {
        is      => 'rw',
        default => '',
    },

    helper_type      => 'Str',
    method_provider  => 'MooseX::AttributeHelpers::MethodProvider::String',
    auto_provide     => 1,
    shortcut         => 'String',
);

no Moose;
no MooseX::AttributeHelpers::Sugar;

1;

__END__

=pod

=head1 NAME

MooseX::AttributeHelpers::String

=head1 SYNOPSIS

  package MyHomePage;
  use Moose;
  use MooseX::AttributeHelpers;
  
  has 'text' => (
      metaclass => 'String',
      is        => 'rw',
      isa       => 'Str',
      default   => sub { '' },
      provides  => {
          append => "add_text",
          replace => "replace_text",
      }
  );

  my $page = MyHomePage->new();
  $page->add_text("foo"); # same as $page->text($page->text . "foo");
  
=head1 DESCRIPTION

This module provides a simple string attribute, to which mutating string
operations can be applied more easily (no need to make an lvalue attribute
metaclass or use temporary variables). Additional methods are provided for
completion.

If your attribute definition does not include any of I<is>, I<isa>,
I<default> or I<provides> but does use the C<String> metaclass,
then this module applies defaults as in the L</SYNOPSIS>
above. This allows for a very basic attribute definition:

  has 'foo' => (metaclass => 'String');
  $obj->append_foo;

=head1 PROVIDED METHODS

The methods for this metaclass are provided by
L<MooseX::AttributeHelpers::MethodProvider::String>.

=head1 BUGS

All complex software has bugs lurking in it, and this module is no 
exception. If you find a bug please either email me, or add the bug
to cpan-RT.

=head1 AUTHOR

Stevan Little E<lt>stevan@iinteractive.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2007-2008 by Infinity Interactive, Inc.

L<http://www.iinteractive.com>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
